<?php

// This is a IA (chatgpt) powered conversational Bot. You must supply your 
// OpenAI API key for it to work. You will also need to modify the SYSTEM_PROMPT
// with data about your company/service/product. This will use the token limit 
// of ChatGPT, so you cannot have a very large prompt. Usually it is limited to
// 4096 tokens counting both the supplied prompt, the history context and the answer.
//
// For larger context we offer a paid bot option that uses RAG to expand knowledge
// almost without limits. Contact us for more info.

$DEBUG        = 0;
$OPENAI_TOKEN = '';
$MODEL        = "gpt-3.5-turbo";

$GUARDRAILS_USER_PROMPT='. Do not justify your answer.';

$SYSTEM_PROMPT = "You are a helpful virtual assistant.";

$ACTION_PROMPT = "If the user is frustrated or angry, or if the user asks to connect to a real person, you will pass the conversation to a human by appending '[transfer]'. In the farewell/goodbye message you will append '[close]'.";

// override all hardcoded options if we have a bot name and ini file
if(isset($_REQUEST['bot'])) {
    $filename = getcwd()."/".$_REQUEST['bot'].".ini";
    if(is_readable($filename)) {
        $ini = parse_ini_file($filename,false);
        $OPENAI_TOKEN = $ini['openai_token'];
        $SYSTEM_PROMPT = $ini['system_prompt'];
        $ACTION_PROMPT = $ini['action_prompt'];
        $GUARDRAILS_USER_PROMPT = $ini['guardrails_prompt'];
        $MODEL = $ini['model'];
    }
}

$json = file_get_contents('php://input');
$data = json_decode($json,1);

log_debug("POST RECEIVED:\n".print_r($data,1));

$user_prompt = $data['body'];

$user_prompt.=$GUARDRAILS_USER_PROMPT; // add guardrails to question

$chat_context = array_slice($data['chat_context'],-5);
$chat_context[] = array("role"=>"user","content"=>$user_prompt);

if($OPENAI_TOKEN=='') {
    $response = "Bot is not yet configured, be sure to set up your OpenAI API key";
} else {
    $response = ask_openai($OPENAI_TOKEN,$SYSTEM_PROMPT." ".$ACTION_PROMPT,$chat_context);
}

$return = array("body"=>$response,"attach"=>'');
log_debug("REPLY: $response");
echo json_encode($return);
die();

function ask_openai($openai_token,$system_prompt,$user_prompt) {
    global $MODEL;

    $url = 'https://api.openai.com/v1/chat/completions';
    $sysprompt = array();
    $sysprompt[]=array("role"=>"system","content"=>$system_prompt);
    $messages = array_merge($sysprompt,$user_prompt);
    $data = array(
       "model"=> $MODEL,
       "messages" => $messages
    );
    $data_string = json_encode($data);
    log_debug("OpenAI Messages: ".print_r($messages,1));
    $headers=array('Content-Type: application/json','Content-Length: ' . strlen($data_string),"Authorization: Bearer $openai_token");
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_TIMEOUT, 120);
    curl_setopt($ch, CURLOPT_VERBOSE, true);
    curl_setopt($ch, CURLOPT_SSLVERSION, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $return = curl_exec($ch);
    curl_close($ch);
    $response = json_decode($return,1);
    log_debug("OpenAI response: ".print_r($response,1));
    return $response['choices'][0]['message']['content'];
}

function log_debug($text) {
    global $DEBUG;
    $logfile = basename(__FILE__,'.php');
    if($DEBUG==1) {
        file_put_contents("/tmp/${logfile}.log",$text."\n",FILE_APPEND);
    }
}
